@@ -31,6 +31,8 @@
 #if defined(HAVE_LIBSTATGRAB) && !defined(HAVE_LIBSTATGRAB_H)
 #undef HAVE_LIBSTATGRAB
 #endif
+#include <assert.h>
+#include <stdint.h>
 
 #ifndef HAVE_LIBSTATGRAB
 #include <linux/kernel.h>
@@ -50,34 +52,41 @@
 #endif
 #define BARLEN 35
 #define HEADERLEN 14
-#define VERSION "0.9.2"
+#define VERSION "0.9.3"
 
 extern char *optarg;
 extern int optind, opterr, optopt;
 
 struct freecolor_meminfo {
-	unsigned long totalram;  /* Total usable main memory size */
-	unsigned long freeram;   /* Available memory size */
-	unsigned long sharedram; /* Amount of shared memory */
-	unsigned long bufferram; /* Memory used by buffers */
-	unsigned long cachedram; /* Memory used for I/O cache */
-	unsigned long totalswap; /* Total swap space size */
-	unsigned long freeswap;  /* swap space still available */
+	uint64_t totalram;  /* Total usable main memory size */
+	uint64_t freeram;   /* Available memory size */
+	uint64_t sharedram; /* Amount of shared memory */
+	uint64_t bufferram; /* Memory used by buffers */
+	uint64_t cachedram; /* Memory used for I/O cache */
+	uint64_t totalswap; /* Total swap space size */
+	uint64_t freeswap;  /* swap space still available */
 };
 
-void bargraph(float percent, float secondper, char marks[BARLEN + HEADERLEN + 1], int usefull) {
+void bargraph(double percent, double secondper, char marks[BARLEN + HEADERLEN + 1], int usefull) {
 	char percentone[BARLEN + 1], percenttwo[BARLEN + 1], remain[BARLEN + 1];
 	unsigned int numberofmarks, numofmarkstwo, remainnum;
 
-	numberofmarks=(int) ((float) (BARLEN * (percent / 100)));
+	numberofmarks=(int) ((double) (BARLEN * (percent / 100)));
+
+	assert(numberofmarks <= BARLEN);
 
 	if (!usefull) {
-		numofmarkstwo = (int) ((float) (BARLEN * (secondper / 100)));
+		numofmarkstwo = (int) ((double) (BARLEN * (secondper / 100)));
 	} else {
 		numofmarkstwo = (BARLEN - numberofmarks);
 	}
 
+	assert(numofmarkstwo <= BARLEN);
+
 	remainnum = BARLEN - (numberofmarks + numofmarkstwo);
+
+	assert(remainnum <= BARLEN);
+
 	memset(percentone, '#', numberofmarks);
 	memset(percenttwo, '%', numofmarkstwo);
 	memset(remain, '.', remainnum);
@@ -123,17 +132,24 @@ unsigned long get_meminfo(char searchfor[12]) {
 
 int get_all_meminfo(struct freecolor_meminfo *retval) {
 	struct sysinfo sinfo;
-	unsigned long meminfo_cached;
+	uint64_t meminfo_cached;
+	uint64_t multiplier;
 
 	meminfo_cached = get_meminfo("Cached:");
 	sysinfo(&sinfo);
 
-	retval->totalram  = sinfo.totalram;
-	retval->freeram   = sinfo.freeram;
-	retval->sharedram = sinfo.sharedram;
-	retval->bufferram = sinfo.bufferram;
-	retval->totalswap = sinfo.totalswap;
-	retval->freeswap  = sinfo.freeswap;
+#ifdef HAVE_SYSINFO_MEM_UNIT
+	multiplier = sinfo.mem_unit;
+#else
+	multiplier = 1;
+#endif
+
+	retval->totalram  = sinfo.totalram * multiplier;
+	retval->freeram   = sinfo.freeram * multiplier;
+	retval->sharedram = sinfo.sharedram * multiplier;
+	retval->bufferram = sinfo.bufferram * multiplier;
+	retval->totalswap = sinfo.totalswap * multiplier;
+	retval->freeswap  = sinfo.freeswap * multiplier;
 
 	retval->cachedram = meminfo_cached;
 
@@ -187,9 +203,9 @@ int get_all_meminfo(struct freecolor_meminfo *retval) {
 
 int main(int argc, char **argv) {
 	struct freecolor_meminfo sinfo;
-	unsigned long cachedbuffer, divisor;
-	float percentram, percentswap, percentbuffer, percentused, percentfree;
-	float ramfree, ramtotal, swapfree, swaptotal, doloop, totaltotal;
+	uint64_t cachedbuffer, divisor;
+	double percentram, percentswap, percentbuffer, percentused, percentfree;
+	double ramfree, ramtotal, swapfree, swaptotal, doloop, totaltotal;
 	char realbarchart[BARLEN  +HEADERLEN + 1], swapbarchart[BARLEN + HEADERLEN + 1], totalbarchart[BARLEN + HEADERLEN + 1];
 	int i, dototals, doold, linestoup;
 	int gam_ret;
@@ -249,13 +265,13 @@ int main(int argc, char **argv) {
 		totaltotal   = sinfo.totalram + sinfo.totalswap;
 		cachedbuffer = sinfo.bufferram + sinfo.cachedram;
 
-		percentram    = (float) ((ramfree / ramtotal) * 100);
-		percentbuffer = (float) ((cachedbuffer / ramtotal) * 100);
-		percentfree   = (float) (((sinfo.freeram + sinfo.freeswap + cachedbuffer) / totaltotal) * 100);
+		percentram    = (double) ((ramfree / ramtotal) * 100);
+		percentbuffer = (double) ((cachedbuffer / ramtotal) * 100);
+		percentfree   = (double) (((sinfo.freeram + sinfo.freeswap + cachedbuffer) / totaltotal) * 100);
 		percentused   = (int) (100 - ((int) percentfree));
 
 		if (swaptotal!=0) {
-			percentswap = (float) (( swapfree / swaptotal) * 100);
+			percentswap = (double) (( swapfree / swaptotal) * 100);
 		} else {
 			percentswap = 0;
 		}
@@ -268,21 +284,21 @@ int main(int argc, char **argv) {
 			printf("Physical  : \033[1;30m[\033[1;32m%s\033[1;30m]\033[1;37m %i\033[0;31m%%\033[0m\t(%lu/%lu)\n",
 			    realbarchart,
 			    (int) (percentram + percentbuffer),
-			    (unsigned long) (ramfree + cachedbuffer) / divisor,
-			    (unsigned long) ramtotal/divisor
+			    (unsigned long) ((ramfree + cachedbuffer) / divisor),
+			    (unsigned long) (ramtotal / divisor)
 			);
 
 			printf("Swap      : \033[1;30m[\033[1;32m%s\033[1;30m]\033[1;37m %i\033[0;31m%%\033[0m\t(%lu/%lu)\n",
 			    swapbarchart,
 			     (int) percentswap,
-			     (unsigned long) swapfree / divisor,
-			     (unsigned long) swaptotal/divisor
+			     (unsigned long) (swapfree / divisor),
+			     (unsigned long) (swaptotal/divisor)
 			);
 
 			if (dototals) {
 				printf("Total     : \033[1;30m[\033[1;32m%s\033[1;30m]\033[1;37m \033[0m(%lu=%lu+%lu)\n",
 				    totalbarchart,
-				    (unsigned long) (sinfo.totalram + sinfo.totalswap) / divisor,
+				    (unsigned long) ((sinfo.totalram + sinfo.totalswap) / divisor),
 				    (unsigned long) ((sinfo.freeram + sinfo.freeswap) / divisor),
 				    (unsigned long) ((totaltotal - ((sinfo.freeram + sinfo.freeswap + cachedbuffer))) / divisor)
 				);
@@ -307,8 +323,8 @@ int main(int argc, char **argv) {
 			if (dototals) {
 				printf("Total: %11lu = (%8lu (used) + %8lu (free))\n",
 				    (unsigned long) (totaltotal / divisor),
-				    (((long) totaltotal) - ((sinfo.freeram+sinfo.freeswap))) / divisor,
-				    (sinfo.freeram+sinfo.freeswap+((unsigned long) cachedbuffer)) / divisor
+				    (unsigned long) ((((long) totaltotal) - ((sinfo.freeram+sinfo.freeswap))) / divisor),
+				    (unsigned long) ((sinfo.freeram+sinfo.freeswap+((unsigned long) cachedbuffer)) / divisor)
 				);
 			}
 		}
